part of katana_routing;

/// Color definition for the theme.
class ThemeColor {
  /// Color definition for the theme.
  const ThemeColor({
    this.primary = Colors.blue,
    this.secondary = Colors.cyan,
    this.primaryVariant = Colors.blueAccent,
    this.secondaryVariant = Colors.cyanAccent,
    this.disabled = Colors.grey,
    this.weak = Colors.grey,
    this.error = Colors.red,
    this.warning = Colors.amber,
    this.info = Colors.blue,
    this.success = Colors.green,
    this.surface = const Color(0xFFFFFFFF),
    this.background = const Color(0xFFFFFFFF),
    this.onPrimary = const Color(0xFFFFFFFF),
    this.onSecondary = const Color(0xFFFFFFFF),
    this.onDisabled = const Color(0xFFFFFFFF),
    this.onSurface = const Color(0xFFFFFFFF),
    this.onBackground = const Color(0xFF212121),
    this.onWeak = const Color(0xFFFFFFFF),
    this.onError = const Color(0xFFFFFFFF),
    this.onInfo = const Color(0xFFFFFFFF),
    this.onSuccess = const Color(0xFF212121),
    this.onWarning = const Color(0xFFFFFFFF),
    this.brightness = Brightness.light,
    this.appBarColor,
    this.scaffoldBackgroundColor,
    this.fontFamily,
  });

  /// Color definition for the theme.
  factory ThemeColor.light({
    Color primary = Colors.blue,
    Color secondary = Colors.cyan,
    Color primaryVariant = Colors.blueAccent,
    Color secondaryVariant = Colors.cyanAccent,
    Color disabled = Colors.grey,
    Color weak = Colors.grey,
    Color error = Colors.red,
    Color warning = Colors.amber,
    Color info = Colors.blue,
    Color success = Colors.green,
    Color surface = const Color(0xFFFFFFFF),
    Color background = const Color(0xFFFFFFFF),
    Color onPrimary = const Color(0xFFFFFFFF),
    Color onSecondary = const Color(0xFFFFFFFF),
    Color onDisabled = const Color(0xFFFFFFFF),
    Color onSurface = const Color(0xFF212121),
    Color onBackground = const Color(0xFF212121),
    Color onWeak = const Color(0xFFFFFFFF),
    Color onError = const Color(0xFFFFFFFF),
    Color onInfo = const Color(0xFFFFFFFF),
    Color onSuccess = const Color(0xFFFFFFFF),
    Color onWarning = const Color(0xFFFFFFFF),
    Color? appBarColor,
    Color? scaffoldBackgroundColor,
    String? fontFamily,
  }) =>
      ThemeColor(
        primary: primary,
        secondary: secondary,
        primaryVariant: primaryVariant,
        secondaryVariant: secondaryVariant,
        disabled: disabled,
        weak: weak,
        error: error,
        warning: warning,
        info: info,
        success: success,
        surface: surface,
        background: background,
        onPrimary: onPrimary,
        onSecondary: onSecondary,
        onDisabled: onDisabled,
        onSurface: onSurface,
        onBackground: onBackground,
        onWeak: onWeak,
        onError: onError,
        onInfo: onInfo,
        onSuccess: onSuccess,
        onWarning: onWarning,
        brightness: Brightness.light,
        fontFamily: fontFamily,
        appBarColor: appBarColor,
        scaffoldBackgroundColor: scaffoldBackgroundColor,
      );

  /// Color definition for the theme.
  factory ThemeColor.dark({
    Color primary = Colors.blue,
    Color secondary = Colors.cyan,
    Color primaryVariant = Colors.blueAccent,
    Color secondaryVariant = Colors.cyanAccent,
    Color disabled = Colors.grey,
    Color weak = Colors.grey,
    Color error = Colors.red,
    Color warning = Colors.amber,
    Color info = Colors.blue,
    Color success = Colors.green,
    Color surface = const Color(0xFF212121),
    Color background = const Color(0xFF212121),
    Color onPrimary = const Color(0xFFFFFFFF),
    Color onSecondary = const Color(0xFFFFFFFF),
    Color onDisabled = const Color(0xFFFFFFFF),
    Color onSurface = const Color(0xFFFFFFFF),
    Color onBackground = const Color(0xFFFFFFFF),
    Color onWeak = const Color(0xFFFFFFFF),
    Color onError = const Color(0xFFFFFFFF),
    Color onInfo = const Color(0xFFFFFFFF),
    Color onSuccess = const Color(0xFFFFFFFF),
    Color onWarning = const Color(0xFFFFFFFF),
    Color? appBarColor,
    Color? scaffoldBackgroundColor,
    String? fontFamily,
  }) =>
      ThemeColor(
        primary: primary,
        secondary: secondary,
        primaryVariant: primaryVariant,
        secondaryVariant: secondaryVariant,
        disabled: disabled,
        weak: weak,
        error: error,
        warning: warning,
        info: info,
        success: success,
        surface: surface,
        background: background,
        onPrimary: onPrimary,
        onSecondary: onSecondary,
        onDisabled: onDisabled,
        onSurface: onSurface,
        onBackground: onBackground,
        onWeak: onWeak,
        onError: onError,
        onInfo: onInfo,
        onSuccess: onSuccess,
        onWarning: onWarning,
        brightness: Brightness.dark,
        fontFamily: fontFamily,
        appBarColor: appBarColor,
        scaffoldBackgroundColor: scaffoldBackgroundColor,
      );

  /// Default font family.
  final String? fontFamily;

  /// Primary color.
  final Color primary;

  /// Secondary color.
  final Color secondary;

  /// Primary color variant.
  final Color primaryVariant;

  /// Secondary color variant.
  final Color secondaryVariant;

  /// The color when disabled.
  final Color disabled;

  /// The color of the outside line and the dividing line.
  final Color weak;

  /// Error color.
  final Color error;

  /// Attention color.
  final Color warning;

  /// Information color.
  final Color info;

  /// Color when successful.
  final Color success;

  /// Background color of dialogs, etc.
  final Color surface;

  /// Background color.
  final Color background;

  /// Primary color text color.
  final Color onPrimary;

  /// Secondary color text color.
  final Color onSecondary;

  /// Invalid text color.
  final Color onDisabled;

  /// Character color of the surface.
  final Color onSurface;

  /// Text color for the background color.
  final Color onBackground;

  /// Text color for lines.
  final Color onWeak;

  /// Text color for errors.
  final Color onError;

  /// Text color for information.
  final Color onInfo;

  /// Text color for success color.
  final Color onSuccess;

  /// Text color for attention color.
  final Color onWarning;

  /// Brightness.
  final Brightness brightness;

  /// App bar color.
  final Color? appBarColor;

  /// Scaffold color.
  final Color? scaffoldBackgroundColor;

  /// Conversion to theme data.
  ThemeData toThemeData() {
    switch (brightness) {
      case Brightness.dark:
        final colorScheme = const ColorScheme.dark().copyWith(
          primary: primary,
          primaryVariant: primaryVariant,
          secondary: secondary,
          secondaryVariant: secondaryVariant,
          surface: surface,
          background: background,
          error: error,
          onPrimary: onPrimary,
          onSecondary: onSecondary,
          onBackground: onBackground,
          onSurface: onSurface,
          onError: onError,
          brightness: brightness,
        );
        final theme = ThemeData.dark();
        return theme.copyWith(
          platform: TargetPlatform.iOS,
          colorScheme: colorScheme,
          splashColor: const Color(0xFFFFFFFF).withOpacity(0.8),
          scaffoldBackgroundColor: scaffoldBackgroundColor,
          textTheme: theme.textTheme.apply(
            bodyColor: onBackground,
            displayColor: onBackground,
            fontFamily: fontFamily,
          ),
          appBarTheme: appBarColor != null
              ? theme.appBarTheme.copyWith(
                  backgroundColor: appBarColor,
                  elevation: appBarColor == Colors.transparent ? 0 : null,
                  foregroundColor:
                      appBarColor == Colors.transparent ? onBackground : null,
                  textTheme: appBarColor == Colors.transparent
                      ? theme.textTheme.copyWith().apply(
                            bodyColor: onBackground,
                            displayColor: onBackground,
                          )
                      : null,
                  iconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                  actionsIconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                )
              : null,
          buttonTheme: theme.buttonTheme.copyWith(
            textTheme: theme.buttonTheme.textTheme,
            buttonColor: primary,
            disabledColor: disabled,
            colorScheme: colorScheme,
          ),
          inputDecorationTheme: theme.inputDecorationTheme.copyWith(
            labelStyle: TextStyle(color: weak),
            helperStyle: TextStyle(color: weak),
            hintStyle: TextStyle(color: weak),
            counterStyle: TextStyle(color: weak),
            errorStyle: TextStyle(color: error),
            prefixStyle: TextStyle(color: onBackground),
            suffixStyle: TextStyle(color: onBackground),
            border: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            disabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            focusedBorder: OutlineInputBorder(
              borderSide: BorderSide(color: primary, width: 2),
            ),
            errorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
            focusedErrorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
          ),
          indicatorColor: secondary,
          brightness: brightness,
          primaryColorBrightness: brightness,
          primaryColor: primary,
          disabledColor: disabled,
          dividerColor: weak,
          errorColor: error,
          backgroundColor: background,
          canvasColor: background,
        );
      default:
        final colorScheme = const ColorScheme.light().copyWith(
            primary: primary,
            primaryVariant: primaryVariant,
            secondary: secondary,
            secondaryVariant: secondaryVariant,
            surface: surface,
            background: background,
            error: error,
            onPrimary: onPrimary,
            onSecondary: onSecondary,
            onBackground: onBackground,
            onSurface: onSurface,
            onError: onError,
            brightness: brightness);
        final theme = ThemeData.light();
        return theme.copyWith(
          platform: TargetPlatform.iOS,
          colorScheme: colorScheme,
          splashColor: const Color(0xFFFFFFFF).withOpacity(0.8),
          scaffoldBackgroundColor: scaffoldBackgroundColor,
          appBarTheme: appBarColor != null
              ? theme.appBarTheme.copyWith(
                  backgroundColor: appBarColor,
                  elevation: appBarColor == Colors.transparent ? 0 : null,
                  foregroundColor:
                      appBarColor == Colors.transparent ? onBackground : null,
                  textTheme: appBarColor == Colors.transparent
                      ? theme.textTheme.copyWith().apply(
                            bodyColor: onBackground,
                            displayColor: onBackground,
                          )
                      : null,
                  iconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                  actionsIconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                )
              : null,
          textTheme: theme.textTheme.apply(
            bodyColor: onBackground,
            displayColor: onBackground,
            fontFamily: fontFamily,
          ),
          buttonTheme: theme.buttonTheme.copyWith(
            buttonColor: primary,
            disabledColor: disabled,
            colorScheme: colorScheme,
          ),
          inputDecorationTheme: theme.inputDecorationTheme.copyWith(
            labelStyle: TextStyle(color: weak),
            helperStyle: TextStyle(color: weak),
            hintStyle: TextStyle(color: weak),
            counterStyle: TextStyle(color: weak),
            errorStyle: TextStyle(color: error),
            prefixStyle: TextStyle(color: onBackground),
            suffixStyle: TextStyle(color: onBackground),
            border: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            disabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            focusedBorder: OutlineInputBorder(
              borderSide: BorderSide(color: primary, width: 2),
            ),
            errorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
            focusedErrorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
          ),
          indicatorColor: secondary,
          brightness: brightness,
          primaryColorBrightness: brightness,
          primaryColor: primary,
          disabledColor: disabled,
          dividerColor: weak,
          errorColor: error,
          backgroundColor: background,
          canvasColor: background,
        );
    }
  }

  /// Convert the theme color from [DynamicMap].
  static ThemeColor? _fromMap(DynamicMap map) {
    if (!map.containsKey("primary") ||
        !map.containsKey("secondary") ||
        !map.containsKey("primaryVariant") ||
        !map.containsKey("secondaryVariant") ||
        !map.containsKey("disabled") ||
        !map.containsKey("weak") ||
        !map.containsKey("error") ||
        !map.containsKey("warning") ||
        !map.containsKey("info") ||
        !map.containsKey("success") ||
        !map.containsKey("surface") ||
        !map.containsKey("background") ||
        !map.containsKey("onPrimary") ||
        !map.containsKey("onSecondary") ||
        !map.containsKey("onDisabled") ||
        !map.containsKey("onSurface") ||
        !map.containsKey("onBackground") ||
        !map.containsKey("onWeak") ||
        !map.containsKey("onError") ||
        !map.containsKey("onInfo") ||
        !map.containsKey("onSuccess") ||
        !map.containsKey("onWarning") ||
        !map.containsKey("brightness")) {
      return null;
    }
    final appBarColor = map.get<int?>("appBarColor", null);
    final scaffoldBackgroundColor =
        map.get<int?>("scaffoldBackgroundColor", null);
    return ThemeColor(
      primary: Color(map.get("primary", 0)),
      secondary: Color(map.get("secondary", 0)),
      primaryVariant: Color(map.get("primaryVariant", 0)),
      secondaryVariant: Color(map.get("secondaryVariant", 0)),
      disabled: Color(map.get("disabled", 0)),
      weak: Color(map.get("weak", 0)),
      error: Color(map.get("error", 0)),
      warning: Color(map.get("warning", 0)),
      info: Color(map.get("info", 0)),
      success: Color(map.get("success", 0)),
      surface: Color(map.get("surface", 0)),
      background: Color(map.get("background", 0)),
      onPrimary: Color(map.get("onPrimary", 0)),
      onSecondary: Color(map.get("onSecondary", 0)),
      onDisabled: Color(map.get("onDisabled", 0)),
      onSurface: Color(map.get("onSurface", 0)),
      onBackground: Color(map.get("onBackground", 0)),
      onWeak: Color(map.get("onWeak", 0)),
      onError: Color(map.get("onError", 0)),
      onInfo: Color(map.get("onInfo", 0)),
      onSuccess: Color(map.get("onSuccess", 0)),
      onWarning: Color(map.get("onWarning", 0)),
      brightness: Brightness.values
          .firstWhere((e) => e.index == map.get("brightness", 0)),
      fontFamily: map.get<String?>("fontFamily", null),
      appBarColor: appBarColor != null ? Color(appBarColor) : null,
      scaffoldBackgroundColor: scaffoldBackgroundColor != null
          ? Color(scaffoldBackgroundColor)
          : null,
    );
  }

  /// Convert the menu config to [DynamicMap].
  DynamicMap toMap() {
    return <String, dynamic>{
      "primary": primary.value,
      "secondary": secondary.value,
      "primaryVariant": primaryVariant.value,
      "secondaryVariant": secondaryVariant.value,
      "disabled": disabled.value,
      "weak": weak.value,
      "error": error.value,
      "warning": warning.value,
      "info": info.value,
      "success": success.value,
      "surface": surface.value,
      "background": background.value,
      "onPrimary": onPrimary.value,
      "onSecondary": onSecondary.value,
      "onDisabled": onDisabled.value,
      "onSurface": onSurface.value,
      "onBackground": onBackground.value,
      "onWeak": onWeak.value,
      "onError": onError.value,
      "onInfo": onInfo.value,
      "onSuccess": onSuccess.value,
      "onWarning": onWarning.value,
      "brightness": brightness.index,
      if (fontFamily.isNotEmpty) "fontFamily": fontFamily,
      if (appBarColor != null) "appBarColor": appBarColor!.value,
      if (scaffoldBackgroundColor != null)
        "scaffoldBackgroundColor": scaffoldBackgroundColor!.value,
    };
  }
}
